@@ -1,5 +1,5 @@
 package WebService::PayPal::PaymentsAdvanced;
-$WebService::PayPal::PaymentsAdvanced::VERSION = '0.000005';
+$WebService::PayPal::PaymentsAdvanced::VERSION = '0.000006';
 use Moo;
 
 use feature qw( say state );
@@ -13,13 +13,25 @@ use Types::URI qw( Uri );
 use URI;
 use URI::FromHash qw( uri uri_object );
 use URI::QueryParam;
-use Web::Scraper;
+
+#<<< don't perltidy
 use WebService::PayPal::PaymentsAdvanced::Error::Generic;
 use WebService::PayPal::PaymentsAdvanced::Error::HostedForm;
 use WebService::PayPal::PaymentsAdvanced::Response;
+use WebService::PayPal::PaymentsAdvanced::Response::Capture;
 use WebService::PayPal::PaymentsAdvanced::Response::FromHTTP;
 use WebService::PayPal::PaymentsAdvanced::Response::FromRedirect;
 use WebService::PayPal::PaymentsAdvanced::Response::FromSilentPOST;
+use WebService::PayPal::PaymentsAdvanced::Response::FromSilentPOST::CreditCard;
+use WebService::PayPal::PaymentsAdvanced::Response::FromSilentPOST::PayPal;
+use WebService::PayPal::PaymentsAdvanced::Response::Inquiry;
+use WebService::PayPal::PaymentsAdvanced::Response::Inquiry::CreditCard;
+use WebService::PayPal::PaymentsAdvanced::Response::Inquiry::PayPal;
+use WebService::PayPal::PaymentsAdvanced::Response::Sale;
+use WebService::PayPal::PaymentsAdvanced::Response::Sale::CreditCard;
+use WebService::PayPal::PaymentsAdvanced::Response::Sale::PayPal;
+use WebService::PayPal::PaymentsAdvanced::Response::SecureToken;
+#>>>
 
 has partner => (
     is       => 'ro',
@@ -52,15 +64,6 @@ has production_mode => (
     default => 0,
 );
 
-has ua => (
-    is      => 'ro',
-    default => sub {
-        my $ua = LWP::UserAgent->new;
-        $ua->timeout(5);
-        return $ua;
-    }
-);
-
 has user => (
     is       => 'ro',
     isa      => Str,
@@ -85,6 +88,11 @@ has verbose => (
     default => 1,
 );
 
+with(
+    'WebService::PayPal::PaymentsAdvanced::Role::HasUA',
+    'WebService::PayPal::PaymentsAdvanced::Role::ClassFor'
+);
+
 sub _build_payflow_pro_uri {
     my $self = shift;
 
@@ -116,6 +124,11 @@ sub capture_delayed_transaction {
     return $self->post( { TRXTYPE => 'D', ORIGID => $pnref } );
 }
 
+sub _class_for {
+    my $self = shift;
+    return 'WebService::PayPal::PaymentsAdvanced::' . shift;
+}
+
 sub create_secure_token {
     my $self = shift;
 
@@ -140,12 +153,9 @@ sub get_response_from_redirect {
     state $check = compile(HashRef);
     my ($args) = $check->(@_);
 
-    my $response
-        = WebService::PayPal::PaymentsAdvanced::Response::FromRedirect->new(
-        $args);
+    my $response = $self->_class_for('Response::FromRedirect')->new($args);
 
-    return WebService::PayPal::PaymentsAdvanced::Response->new(
-        params => $response->params );
+    return $self->_class_for('Response')->new( params => $response->params );
 }
 
 sub get_response_from_silent_post {
@@ -154,15 +164,33 @@ sub get_response_from_silent_post {
     state $check = compile(HashRef);
     my ($args) = $check->(@_);
 
-    my $response
-        = WebService::PayPal::PaymentsAdvanced::Response::FromSilentPOST
-        ->new($args);
+    # If the RESPMSG param is missing, then this may just be a garbage, random
+    # POST from a bot.
 
-    return WebService::PayPal::PaymentsAdvanced::Response->new(
-        params => $response->params );
+    unless ( $args->{params}->{RESPMSG} ) {
+        WebService::PayPal::PaymentsAdvanced::Error::Generic->throw(
+            message => 'Bad params supplied from silent POST',
+            params  => $args->{params},
+        );
+    }
+
+    # First we create a SilentPOST response, which may or may not validate the
+    # IP. If there's no exception we query the object to find out if this was a
+    # PayPal or CreditCard transaction and then return the appropriate class.
+    # IPs will only be validate once as the PayPal/CreditCard object
+    # instantiation will not provide an IP address.
+
+    my $response_class = $self->_class_for('Response::FromSilentPOST');
+    my $response       = $response_class->new($args);
+
+    $response_class
+        .= '::'
+        . ( $response->is_credit_card_transaction ? 'CreditCard' : 'PayPal' );
+
+    return $response_class->new( params => $response->params );
 }
 
-sub transaction_status {
+sub inquiry_transaction {
     my $self = shift;
 
     state $check = compile(Str);
@@ -177,47 +205,6 @@ sub transaction_status {
     );
 }
 
-sub hosted_form_uri {
-    my $self = shift;
-
-    state $check = compile(
-        InstanceOf ['WebService::PayPal::PaymentsAdvanced::Response'] );
-    my ($response) = $check->(@_);
-
-    my $uri = $self->payflow_link_uri->clone;
-    $uri->query_param( SECURETOKEN   => $response->secure_token, );
-    $uri->query_param( SECURETOKENID => $response->secure_token_id, );
-
-    return $uri unless $self->validate_hosted_form_uri;
-
-    # For whatever reason on the PayPal side, HEAD isn't useful here.
-    my $res = $self->ua->get($uri);
-
-    unless ( $res->is_success ) {
-
-        WebService::PayPal::PaymentsAdvanced::Error::HTTP->throw(
-            message       => "hosted_form URI does not validate: $uri",
-            http_response => $res,
-            http_status   => $res->code,
-        );
-
-    }
-
-    my $error_scraper = scraper {
-        process( '.error', error => 'TEXT' );
-    };
-
-    my $scraped_text = $error_scraper->scrape($res);
-
-    return $uri unless exists $scraped_text->{error};
-
-    WebService::PayPal::PaymentsAdvanced::Error::HostedForm->throw(
-        message =>
-            "hosted_form contains error message: $scraped_text->{error}",
-        http_response => $res,
-    );
-}
-
 sub post {
     my $self = shift;
 
@@ -234,11 +221,48 @@ sub post {
         = $self->ua->post( $self->payflow_pro_uri, Content => $content );
 
     my $params
-        = WebService::PayPal::PaymentsAdvanced::Response::FromHTTP->new(
-        http_response => $http_response )->params;
+        = $self->_class_for('Response::FromHTTP')
+        ->new( http_response => $http_response )->params;
 
-    return WebService::PayPal::PaymentsAdvanced::Response->new(
-        params => $params );
+    my $response_class = $self->_class_for('Response');
+
+    if ( $post->{CREATESECURETOKEN} && $post->{CREATESECURETOKEN} eq 'Y' ) {
+        return $self->_class_for('Response::SecureToken')->new(
+            params                   => $params,
+            payflow_link_uri         => $self->payflow_link_uri,
+            ua                       => $self->ua,
+            validate_hosted_form_uri => $self->validate_hosted_form_uri,
+        );
+    }
+
+    if ( $post->{TRXTYPE} ) {
+        if ( $post->{TRXTYPE} eq 'D' ) {
+            $response_class = $self->_class_for('Response::Capture');
+        }
+        if ( $post->{TRXTYPE} eq 'I' ) {
+            $response_class = $self->_class_for('Response::Inquiry');
+            my $response = $response_class->new( params => $params );
+            $response_class .= '::'
+                . (
+                $response->is_credit_card_transaction
+                ? 'CreditCard'
+                : 'PayPal'
+                );
+        }
+        if ( $post->{TRXTYPE} eq 'S' ) {
+            $response_class = $self->_class_for('Response::Sale');
+            my $response = $response_class->new( params => $params );
+            $response_class .= '::'
+                . (
+                $response->is_credit_card_transaction
+                ? 'CreditCard'
+                : 'PayPal'
+                );
+        }
+        return $response_class->new( params => $params );
+    }
+
+    return $response_class->new( params => $params );
 }
 
 sub sale_from_credit_card_reference_transaction {
@@ -327,15 +351,14 @@ sub _force_upper_case {
 }
 
 # Payments Advanced treats encoding key/value pairs like a special snowflake.
-# https://metacpan.org/source/PLOBBES/Business-OnlinePayment-PayPal::PaymentsAdvanced-1.01/PayPal::PaymentsAdvanced.pm#L276
-
+# https://metacpan.org/source/PLOBBES/Business-OnlinePayment-PayflowPro-1.01/PayflowPro.pm#L276
 sub _pseudo_encode_args {
     my $self = shift;
     my $args = shift;
 
     my $uri = join '&', map {
         join '=', sprintf( '%s[%i]', $_, length( $args->{$_} ) ), $args->{$_}
-    } sort keys %{$args};
+    } grep { defined $args->{$_} } sort keys %{$args};
     return $uri;
 }
 
@@ -351,7 +374,7 @@ WebService::PayPal::PaymentsAdvanced - A simple wrapper around the PayPal Paymen
 
 =head1 VERSION
 
-version 0.000005
+version 0.000006
 
 =head1 SYNOPSIS
 
@@ -377,7 +400,7 @@ version 0.000005
         }
     );
 
-    my $uri = $payments->hosted_form_uri( $response );
+    my $uri = $response->hosted_form_uri;
 
     # Store token data for later use.  You'll need to implement this yourself.
     $foo->freeze_token_data(
@@ -508,7 +531,7 @@ to C<true>.
 =head3 create_secure_token
 
 Create a secure token which you can use to create a hosted form uri.  Returns a
-L<WebService::PayPal::PaymentsAdvanced::Response> object.
+L<WebService::PayPal::PaymentsAdvanced::Response::SecureToken> object.
 
     use WebService::PayPal::PaymentsAdvanced;
     my $payments = WebService::PayPal::PaymentsAdvanced->new(...);
@@ -543,33 +566,29 @@ L<WebService::PayPal::PaymentsAdvanced::Response> object.
 =head3 get_response_from_silent_post
 
 This method can be used to validate responses from PayPal to your silent POST
-url.  It's essentially a wrapper around
-L<WebService::PayPal::PaymentsAdvanced::Response::FromSilentPost>.  If you
-provide an ip_address parameter, it will be validated against a list of known
-IPs which PayPal provides.  You're encouraged to provide an IP address in order
-to prevent spoofing of payment responses.  See
+url.  If you provide an ip_address parameter, it will be validated against a
+list of known IPs which PayPal provides.  You're encouraged to provide an IP
+address in order to prevent spoofing of payment responses.  See
 L<WebService::PayPal::PaymentsAdvanced::Response::FromSilentPOST> for more
 information on this behaviour.
 
-This method returns a L<WebService::PayPal::PaymentsAdvanced::Response> object.
+This method returns a
+L<WebService::PayPal::PaymentsAdvanced::Response::FromSilentPost::PayPal>
+object for PayPal transactions.  It returns a
+L<WebService::PayPal::PaymentsAdvanced::Response::FromSilentPost::CreditCard>
+object for credit card transactions.  You can either inspect the class return
+to you or use the C<is_credit_card_transaction> or C<is_paypal_transaction>
+methods to learn which method the customer paid with.  Both methods return a
+C<Boolean>.
 
     my $response = $payments->get_response_from_redirect(
         ip_address => $ip,
         params     => $params,
     );
-    print $response->message;
-
-=head3 hosted_form_uri
-
-Returns a L<URI> object which you can use either to insert an iframe into your
-pages or redirect the user to PayPal directly in order to make a payment.
-
-    use WebService::PayPal::PaymentsAdvanced;
-    my $payments = WebService::PayPal::PaymentsAdvanced->new(
-        validate_hosted_form_uri => 1, ... );
-
-    my $response = $payments->create_secure_token(...);
-    my $uri      = $payments->hosted_form_uri($response);
+    print $response->message. "\n";
+    if ( $response->is_credit_card_transaction ) {
+        print $response->card_type, q{ }, $response->card_expiration;
+    }
 
 =head3 post
 
@@ -618,7 +637,12 @@ arguments: a BAID from a previous PayPal transaction, an amount and a currency.
     );
     say $response->message;
 
-=head2 void_transaction( $ORIGID )
+=head3 inquiry_transaction( $ORIGID )
+
+Performs a transaction inquiry on a previously submitted transaction.  Requires
+the ID of the original transaction.  Returns a response object.
+
+=head3 void_transaction( $ORIGID )
 
 Voids a previous transaction.  Requires the ID of the transaction to void.
 Returns a response object.
