@@ -1,32 +1,27 @@
 package WebService::PayPal::PaymentsAdvanced::Response;
-$WebService::PayPal::PaymentsAdvanced::Response::VERSION = '0.000005';
+$WebService::PayPal::PaymentsAdvanced::Response::VERSION = '0.000006';
 use Moo;
 
-use Types::Standard qw( HashRef Str );
+use Types::Standard qw( Str );
 use WebService::PayPal::PaymentsAdvanced::Error::Authentication;
+use WebService::PayPal::PaymentsAdvanced::Error::Generic;
 
-has params => (
-    is       => 'ro',
-    isa      => HashRef,
-    required => 1,
+has pnref => (
+    is      => 'lazy',
+    isa     => Str,
+    default => sub { shift->params->{PNREF} },
 );
 
-has message => (
-    is       => 'lazy',
-    isa      => Str,
-    init_arg => undef,
+has ppref => (
+    is      => 'lazy',
+    isa     => Str,
+    default => sub { shift->params->{PPREF} },
 );
 
-has secure_token => (
-    is       => 'lazy',
-    isa      => Str,
-    init_arg => undef,
-);
-
-has secure_token_id => (
-    is       => 'lazy',
-    isa      => Str,
-    init_arg => undef,
+with(
+    'WebService::PayPal::PaymentsAdvanced::Role::ClassFor',
+    'WebService::PayPal::PaymentsAdvanced::Role::HasParams',
+    'WebService::PayPal::PaymentsAdvanced::Role::HasMessage',
 );
 
 sub BUILD {
@@ -37,33 +32,18 @@ sub BUILD {
     return if defined $result && !$result;
 
     if ( $result && $result == 1 ) {
-        WebService::PayPal::PaymentsAdvanced::Error::Authentication->throw(
+        $self->_class_for('Error::Authentication')->throw(
             message => 'Authentication error: ' . $self->message,
             params  => $self->params,
         );
     }
 
-    WebService::PayPal::PaymentsAdvanced::Error::Generic->throw(
+    $self->_class_for('Error::Generic')->throw(
         message => $self->message,
         params  => $self->params,
     );
 }
 
-sub _build_message {
-    my $self = shift;
-    return $self->params->{RESPMSG};
-}
-
-sub _build_secure_token {
-    my $self = shift;
-    return $self->params->{SECURETOKEN};
-}
-
-sub _build_secure_token_id {
-    my $self = shift;
-    return $self->params->{SECURETOKENID};
-}
-
 1;
 
 =pod
@@ -76,7 +56,7 @@ WebService::PayPal::PaymentsAdvanced::Response - Generic response object
 
 =head1 VERSION
 
-version 0.000005
+version 0.000006
 
 =head1 SYNOPSIS
 
@@ -87,28 +67,27 @@ version 0.000005
 =head1 DESCRIPTION
 
 This module provides a consistent interface for getting information from a
-PayPal response, regardless of whether it comes from token creation, a
-redirect or a silent POST.
+PayPal response, regardless of whether it comes from token creation, a redirect
+or a silent POST.  It will be used as a parent class for other response
+classes.  You should never need to create this object yourself.
 
-=head1 OBJECT INSTANTIATION
+=head1 METHODS
 
-The following parameters can be supplied to C<new()> when creating a new object.
+=head2 message
 
-=head2 Required Parameters
+The contents of PayPal's RESPMSG parameter.
 
-=head3 params
+=head2 params
 
 A C<HashRef> of parameters which have been returned by PayPal.
 
-=head2 secure_token
+=head2 pnref
 
-Returns the PayPal SECURETOKEN param.
+The contents of PayPal's PNREF parameter.
 
-=head2 secure_token_id
+=head2 ppref
 
-Returns the PayPal SECURETOKENID param.  If you are using this module
-directly, you should check that the returned value is the same as the
-SECURETOKENID which you initially provided to PayPal.
+The contents of PayPal's PNREF parameter.
 
 =head1 SEE ALSO
 
