@@ -1,8 +1,11 @@
 package WebService::PayPal::PaymentsAdvanced::Response::FromRedirect;
-$WebService::PayPal::PaymentsAdvanced::Response::FromRedirect::VERSION = '0.000005';
+$WebService::PayPal::PaymentsAdvanced::Response::FromRedirect::VERSION = '0.000006';
 use Moo;
 
-with 'WebService::PayPal::PaymentsAdvanced::Role::HasParams';
+with(
+    'WebService::PayPal::PaymentsAdvanced::Role::HasParams',
+    'WebService::PayPal::PaymentsAdvanced::Role::HasMessage',
+);
 
 1;
 
@@ -16,7 +19,7 @@ WebService::PayPal::PaymentsAdvanced::Response::FromRedirect - Response object f
 
 =head1 VERSION
 
-version 0.000005
+version 0.000006
 
 =head1 DESCRIPTION
 
@@ -45,6 +48,10 @@ redirect or a silent POST.
 Returns the same C<HashRef> of parameters which was initially provided to the
 C<new> method.
 
+=head2 message
+
+Returns the value of the C<RESPMSG> param.
+
 =head1 AUTHOR
 
 Olaf Alders <olaf@wundercounter.com>
