@@ -1,5 +1,5 @@
 package WebService::PayPal::PaymentsAdvanced::Response::FromSilentPOST;
-$WebService::PayPal::PaymentsAdvanced::Response::FromSilentPOST::VERSION = '0.000005';
+$WebService::PayPal::PaymentsAdvanced::Response::FromSilentPOST::VERSION = '0.000006';
 use Moo;
 
 use List::AllUtils qw( any );
@@ -8,7 +8,7 @@ use MooX::StrictConstructor;
 use Types::Standard qw( Bool HashRef Str );
 use WebService::PayPal::PaymentsAdvanced::Error::IPVerification;
 
-with 'WebService::PayPal::PaymentsAdvanced::Role::HasParams';
+extends 'WebService::PayPal::PaymentsAdvanced::Response';
 
 sub BUILD {
     my $self = shift;
@@ -55,6 +55,12 @@ has _ip_addresses => (
     },
 );
 
+with(
+    'WebService::PayPal::PaymentsAdvanced::Role::HasTender',
+    'WebService::PayPal::PaymentsAdvanced::Role::HasTokens',
+    'WebService::PayPal::PaymentsAdvanced::Role::HasTransactionTime',
+);
+
 sub _build_ip_address_is_verified {
     my $self = shift;
 
@@ -73,16 +79,19 @@ WebService::PayPal::PaymentsAdvanced::Response::FromSilentPOST - Response object
 
 =head1 VERSION
 
-version 0.000005
+version 0.000006
 
 =head1 DESCRIPTION
 
 This module provides an interface for extracting returned params from an
 L<HTTP::Response> object.  You won't need to this module directly if you are
-using L<PayPal::PaymentsAdvanced/create_secure_token>.
+using L<PayPal::PaymentsAdvanced/get_response_from_silent_post>.
 
-Throws a L<WebService::PayPal::PaymentsAdvanced::Error::HTTP> exception if the
-HTTP request was not successful.
+This module inherits from L<WebService::PayPal::PaymentsAdvanced::Response> and
+includes the methods provided by
+L<WebService::PayPal::PaymentsAdvanced::Role::HasTender>,
+L<WebService::PayPal::PaymentsAdvanced::Role::HasTokens> and
+L<WebService::PayPal::PaymentsAdvanced::Role::HasTransactionTime>.
 
 =head1 OBJECT INSTANTIATION
 
@@ -108,13 +117,6 @@ This module will throw a
 L<WebService::PayPal::PaymentsAdvanced::Error::IPVerification> exception if
 the provided IP address cannot be validated.
 
-=head2 Methods
-
-=head3 params
-
-Returns the same C<HashRef> of parameters which was initially provided to the
-C<new> method.
-
 =head1 AUTHOR
 
 Olaf Alders <olaf@wundercounter.com>
